#! /usr/bin/perl
#
# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006-2007 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


use strict;
use warnings FATAL => 'all';
use diagnostics;

use Wanha::DB;
use Wanha::Common;
use Wanha::HTTP;

use Wanha::Link;
use Wanha::Chan;

use Wanha::Config;


=item get_latest
Get the latest URLs

0: chan
1: count
r: array of hash (url_id, url_url, url_count, url_date, chan_id, chan_name,
			nick_id, nick_name)
=cut
sub get_latest($$) {
	my ($chan_id, $count) = @_;
	my ($query, $result);
	my @ret;

	$count = &Wanha::Common::min($count, 100);
	my $date_fmt = 'YYYY-MM-DD"T"HH24:MI:SS.MS"Z"';
	my $to_char = "TO_CHAR(urls.time, '$date_fmt')";
	if (defined($chan_id)) {
		$query = &Wanha::DB::prepare(
				"SELECT urls.id, urls.url, urls.count, " .
					"$to_char AS time, " .
					"chans.id, chans.chan, " .
					"nicks.id, nicks.nick " .
				"FROM urls, chans, nicks " .
				"WHERE chans.id = &da& ".
					"AND urls.nick = nicks.id " .
					"AND urls.chan = chans.id " .
				"ORDER BY urls.id DESC " .
				"LIMIT &db&");
		$query = &Wanha::DB::set($query,
				"a", $chan_id,
				"b", $count);
	} else {
		$query = &Wanha::DB::prepare(
				"SELECT urls.id, urls.url, urls.count, " .
					"$to_char AS time, " .
					"chans.id, chans.chan, " .
					"nicks.id, nicks.nick " .
				"FROM urls, chans, nicks " .
				"WHERE urls.nick = nicks.id " .
					"AND urls.chan = chans.id " .
				"ORDER BY urls.id DESC " .
				"LIMIT &da&");
		$query = &Wanha::DB::set($query, "a", $count);
	}
	$result = &Wanha::DB::query($query);
	(&Wanha::DB::row_count($result) == 0) && return ();

	while (my ($url_id, $url_url, $url_count, $url_date,
				$chan_id, $chan_name,
				$nick_id, $nick_name) =
			&Wanha::DB::get_row($result)) {
		my %t = (
			'url_id' => $url_id,
			'url_url' => $url_url,
			'url_date' => $url_date,
			'url_count' => $url_count,
			'chan_id' => $chan_id,
			'chan_name' => $chan_name,
			'nick_id' => $nick_id,
			'nick_name' => $nick_name
			);
		push(@ret, \%t);
	}
	return @ret;
}


=item get_id
Get feed item id

0: type: 0 = feed, 1 = entry
1: channel (can be undef)
2: latest url_id
r: atom::id
=cut
sub get_id($$$) {
	my ($type, $chan, $url_id) = @_;
	my $uri = $Wanha::Config::base_url;
	$uri =~ s,https?://,, if ($uri =~ /^http/);
	$uri =~ s:/+:,:g;
	my $tag = defined($chan) ? "chan-$chan" : "global";
	if ($type == 0) {
		return "tag:$uri:feed-$tag";
	} else {
		return "tag:$uri:entry-$tag-url-$url_id";
	}
}


=item print_entries
Print feed item entries

0: chan id
=cut
sub print_entries($@) {
	my $chan = shift;
	foreach my $t (@_) {
		my %link = %{($t)};

		my $url = &Wanha::Link::amp_url($link{'url_url'});
		my $title = &Wanha::Link::amp_url(&Wanha::HTML::truncate_url(
				$link{'url_url'}));
		my $id = &get_id(1, $chan, $link{'url_id'});
		print <<EOF;
<entry>
	<title>$title</title>
	<link href="$url" />
	<id>$id</id>
	<updated>$link{'url_date'}</updated>
	<summary>Pasted on $link{'chan_name'} by $link{'nick_name'}.
		Wanha count: $link{'url_count'}</summary>
</entry>
EOF
	}
}


=item main
Main function
=cut
sub main() {
	my $chan = &Wanha::HTTP::get_numeric("chan");
	my @urls = &get_latest($chan, 50);

	my $title;
	if (defined($chan)) {
		my $chan = &Wanha::Chan::get_name($chan);
		$title = "wanha links for channel $chan";
	} else {
		$title = "wanha links";
	}

	my @date = localtime();
	my $now = sprintf("%04d-%02d-%02dT%02d:%02d:%02dZ",
			$date[5] + 1900,	# YYYY
			$date[4],		# MM
			$date[3],		# DD
			$date[2],		# HH24
			$date[1],		# MI
			$date[0]);		# SS

	my $id = &get_id(0, $chan, $urls[0]{'url_id'});
	my $gen = &Wanha::Link::amp_url($Wanha::Config::base_url);

	print <<EOF;
Content-Type: application/atom+xml; charset=utf-8

<?xml version="1.0" encoding="UTF-8"?>
<feed xmlns="http://www.w3.org/2005/Atom">

<title>$title</title>
<link href="$Wanha::Config::base_url" />
<updated>$now</updated>
<author>
	<name>Tommi Saviranta</name>
	<address>wnd\@iki.fi</address>
</author>
<id>$id</id>
<generator uri="$gen">wanha</generator>

EOF

	&print_entries($chan, @urls);

	print "</feed>\n";
}


&main();
